"use strict";
/// <reference lib="dom" />
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetchAdapter = void 0;
var axios_1 = require("axios");
var fetch_utils_1 = require("./fetch.utils");
/**
 * fetch adapter for axios
 * Contrary to several other adapters, this one does not rely on axios internal helpers that needs contant refactoring and maintenance
 * Instead, it implements some basic helpers itself to make it work with axios. This will make this adapter easier to maintain and extend
 * @param conf AxiosFetchRequestConfig
 * @returns
 */
var fetchAdapter = function (conf) { return __awaiter(void 0, void 0, void 0, function () {
    var config, request, response, axiosResponse;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                config = conf;
                request = createFetchRequest(config);
                return [4 /*yield*/, fetchRequest(request, config)];
            case 1:
                response = _a.sent();
                return [4 /*yield*/, getAxiosResponse(request, response, config)];
            case 2:
                axiosResponse = _a.sent();
                if (config.validateStatus && !config.validateStatus(axiosResponse.status)) {
                    throw new axios_1.AxiosError(axiosResponse.statusText, "".concat(axiosResponse.status), config, request, axiosResponse);
                }
                return [2 /*return*/, axiosResponse];
        }
    });
}); };
exports.fetchAdapter = fetchAdapter;
// istanbul ignore next
function createFetchRequest(config) {
    var _a, _b, _c, _d, _e;
    if ((0, fetch_utils_1.isBrowser)() &&
        ((0, fetch_utils_1.isFormData)(config.data) || (0, fetch_utils_1.isBlob)(config.data) || (0, fetch_utils_1.isFile)(config.data)) &&
        ((_a = config.headers) === null || _a === void 0 ? void 0 : _a["Content-Type"])) {
        delete config.headers["Content-Type"];
    }
    var headers = new Headers(config.headers);
    if (config.auth) {
        var username = config.auth.username || "";
        var password = config.auth.password
            ? decodeURI(encodeURIComponent(config.auth.password))
            : "";
        headers.set("Authorization", "Basic ".concat(btoa(username + ":" + password)));
    }
    if (config.xsrfHeaderName && config.xsrfCookieName) {
        var xsrfValue = config.withCredentials || config.credentials !== "omit"
            ? (0, fetch_utils_1.findCookieByName)(config.xsrfCookieName)
            : undefined;
        if (xsrfValue) {
            headers.set(config.xsrfHeaderName, xsrfValue);
        }
    }
    var fetchOptions = {
        method: ((_b = config.method) === null || _b === void 0 ? void 0 : _b.toUpperCase()) || "GET",
        headers: headers,
        body: config.data,
        signal: config.signal,
        mode: config.mode,
        cache: config.cache,
        integrity: config.integrity,
        keepalive: config.keepalive,
        redirect: ((_c = config.redirect) !== null && _c !== void 0 ? _c : config.maxRedirects === 0) ? "manual" : "follow",
        referrer: config.referrer,
        referrerPolicy: config.referrerPolicy,
        credentials: ((_d = config.credentials) !== null && _d !== void 0 ? _d : config.withCredentials === undefined)
            ? "same-origin"
            : config.withCredentials
                ? "include"
                : "omit",
    };
    var url = (0, fetch_utils_1.combineURLParams)((_e = (0, fetch_utils_1.getFullURL)(config)) !== null && _e !== void 0 ? _e : "/", (0, fetch_utils_1.getSearchParams)(config));
    return new Request(url, fetchOptions);
}
/**
 * Simple wrapper for fetch request that transforms fetch errors into axios errors
 * @param request - fetch Request
 * @param config - AxiosFetchRequestConfig
 * @returns fetch Response
 */
function fetchRequest(request, config) {
    return __awaiter(this, void 0, void 0, function () {
        var error_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    _a.trys.push([0, 2, , 3]);
                    return [4 /*yield*/, fetch(request)];
                case 1: return [2 /*return*/, _a.sent()];
                case 2:
                    error_1 = _a.sent();
                    // istanbul ignore next
                    if (error_1 instanceof Error) {
                        throw new axios_1.AxiosError(error_1.message, axios_1.AxiosError.ERR_NETWORK, config, request);
                    }
                    else {
                        throw new axios_1.AxiosError("Network Error", axios_1.AxiosError.ERR_NETWORK, config, request);
                    }
                    return [3 /*break*/, 3];
                case 3: return [2 /*return*/];
            }
        });
    });
}
/**
 * Simple wrapper for fetch response that transforms fetch headers and response format into axios one
 * @param request - A fetch Request
 * @param response - A fetch Response
 * @param config - AxiosFetchRequestConfig
 * @returns transformed axios response
 */
function getAxiosResponse(request, response, config) {
    return __awaiter(this, void 0, void 0, function () {
        var headers, result;
        var _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    headers = {};
                    response.headers.forEach(function (value, name) {
                        headers[name.toLowerCase()] = value;
                    });
                    _a = {
                        request: request,
                        headers: headers,
                        config: config,
                        status: response.status,
                        statusText: response.statusText
                    };
                    return [4 /*yield*/, getResponseData(response, config)];
                case 1:
                    result = (_a.data = _b.sent(),
                        _a);
                    return [2 /*return*/, result];
            }
        });
    });
}
// istanbul ignore next
function getResponseData(response, config) {
    var _a;
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_b) {
            switch (config.responseType) {
                case "arraybuffer":
                    return [2 /*return*/, response.arrayBuffer()];
                case "blob":
                    return [2 /*return*/, response.blob()];
                case "document":
                    throw new Error("document response type not supported for fetch");
                case "stream":
                    return [2 /*return*/, response.body];
                case "text":
                    return [2 /*return*/, response.text()];
                case "json":
                    return [2 /*return*/, response.json()];
                default: {
                    if ((_a = response.headers.get("content-type")) === null || _a === void 0 ? void 0 : _a.includes("application/json")) {
                        return [2 /*return*/, response.json()];
                    }
                    return [2 /*return*/, response.text()];
                }
            }
            return [2 /*return*/];
        });
    });
}
