"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.combineURLParams = exports.findCookieByName = exports.getFullURL = exports.getSearchParams = exports.isFile = exports.isBlob = exports.isFormData = exports.isKindOf = exports.isBrowser = void 0;
// istanbul ignore next
function isBrowser() {
    return (typeof window !== "undefined" && typeof window.document !== "undefined");
}
exports.isBrowser = isBrowser;
/**
 * more resilient alternative to instanceof when the object is not native and compiled to es5
 * @param kind - string representation of the object kind
 * @returns - a function that checks if the object is of the given kind
 */
// istanbul ignore next
var isKindOf = function (kind) {
    return function (data) {
        var pattern = "[object ".concat(kind, "]");
        return (data &&
            (toString.call(data) === pattern ||
                (typeof data.toString === "function" && data.toString() === pattern)));
    };
};
exports.isKindOf = isKindOf;
// istanbul ignore next
exports.isFormData = (0, exports.isKindOf)("FormData");
// istanbul ignore next
exports.isBlob = (0, exports.isKindOf)("Blob");
// istanbul ignore next
exports.isFile = (0, exports.isKindOf)("File");
function getSearchParams(config) {
    if (config.params) {
        return config.paramsSerializer
            ? config.paramsSerializer(config.params)
            : new URLSearchParams(config.params).toString();
    }
    return undefined;
}
exports.getSearchParams = getSearchParams;
function getFullURL(config) {
    var _a, _b;
    // istanbul ignore next
    if (((_a = config.url) === null || _a === void 0 ? void 0 : _a.startsWith("http")) || !config.baseURL) {
        return config.url;
    }
    var baseURL = config.baseURL.replace(/\/$/, "");
    var path = (_b = config.url) === null || _b === void 0 ? void 0 : _b.replace(/^\//, "");
    // istanbul ignore next
    if (!path) {
        return baseURL;
    }
    return "".concat(baseURL, "/").concat(path);
}
exports.getFullURL = getFullURL;
// istanbul ignore next
function findCookieByName(name) {
    if (typeof document !== "undefined") {
        var match = document.cookie.match(new RegExp("(^|;\\s*)(" + name + ")=([^;]*)"));
        return match ? decodeURIComponent(match[3]) : undefined;
    }
    return undefined;
}
exports.findCookieByName = findCookieByName;
function combineURLParams(url, searchParams) {
    if (searchParams) {
        // istanbul ignore next
        return url + (url.indexOf("?") === -1 ? "?" : "&") + searchParams;
    }
    return url;
}
exports.combineURLParams = combineURLParams;
